@@ -70,23 +70,6 @@ def encode_utf8(value):
     return value
 
 
-def escape(value):
-    """Escapes a value, note that this is documented for AD queries, but 
-    not for OpenLDAP etc, but hopefully they work in the same manner.
-    """
-    # don't know how to 'find' NUL = \\0
-    #'*' :'\\2a',
-    replacements = {
-        '(' :'\\28',
-        ')' :'\\29',
-        '\\':'\\5c',
-        '/' :'\\2f',
-    }
-    for key, val in replacements.items():
-        value = value.replace(key, val)
-    return value
-
-
 class LDAPConnector(object):
     """Object is responsible for the LDAP connection.
 
@@ -131,7 +114,6 @@ class LDAPConnector(object):
             self._cache = props.cache
             self._cachetimeout = props.timeout
             self._start_tls = props.start_tls
-            #self._escape_queries = props.escape_queries
 
     def bind(self):
         """Bind to Server and return the Connection Object.
@@ -192,7 +174,8 @@ class LDAPCommunicator(object):
         self._con = None
 
     def search(self, queryFilter, scope, baseDN=None,
-               force_reload=False, attrlist=None, attrsonly=0):
+               force_reload=False, attrlist=None, attrsonly=0,
+               page_size=None, cookie=None):
         """Search the directory.
 
         queryFilter
@@ -213,27 +196,60 @@ class LDAPCommunicator(object):
         attrsonly
             Flag whether to return only attribute names, without corresponding
             values.
+
+        page_size
+            Number of items per page, when doing pagination.
+
+        cookie
+            Cookie string returned by previous search with pagination.
         """
         if baseDN is None:
             baseDN = self.baseDN
             if not baseDN:
                 raise ValueError(u"baseDN unset.")
+
+        if page_size:
+            if cookie is None:
+                cookie = ''
+            pagedresults = ldap.controls.libldap.SimplePagedResultsControl(
+                criticality=True, size=page_size, cookie=cookie
+                )
+            serverctrls = [pagedresults,]
+        else:
+            if cookie:
+                raise ValueError('cookie passed without page_size')
+            serverctrls = []
+
+        def _search(baseDN, scope, queryFilter,
+                    attrlist, attrsonly, serverctrls):
+            # we have to do async search to also retrieve server controls
+            # in case we do pagination of results
+            msgid = self._con.search_ext(baseDN, scope, queryFilter,
+                                         attrlist, attrsonly,
+                                         serverctrls=serverctrls)
+            rtype, results, rmsgid, rctrls = self._con.result3(msgid)
+            ctype = ldap.controls.libldap.SimplePagedResultsControl.controlType
+            pctrls = [c for c in rctrls if c.controlType == ctype]
+            if pctrls:
+                return results, pctrls[0].cookie
+            else:
+                return results
         
-        #if self._connector._escape_queries:
-        #    queryFilter = self._escape_query(queryFilter)
-        
+        args = [baseDN, scope, queryFilter, attrlist, attrsonly, serverctrls]
         if self._cache:
-            # XXX: Consider attrlist and attrsonly in cachekey.
-            key = '%s-%s-%s-%i' % (self._connector._bindDN,
+            key = '%s-%s-%s-%s-%s-%i-%s-%s' % (self._connector._bindDN,
                                    baseDN,
+                                   sorted(attrlist or []),
+                                   attrsonly,
                                    queryFilter,
-                                   scope)
+                                   scope,
+                                   page_size,
+                                   cookie)
             key = md5digest(key)
-            args = [baseDN, scope, queryFilter, attrlist, attrsonly]
-            return self._cache.getData(self._con.search_s, key,
+            return self._cache.getData(_search, key,
                                        force_reload, args)
-        return self._con.search_s(baseDN, scope, queryFilter,
-                                  attrlist, attrsonly)
+        else:
+            return _search(*args)
 
     def add(self, dn, data):
         """Insert an entry into directory.
