@@ -1,15 +1,17 @@
 # -*- coding: utf-8 -*-
-from zope.interface import implements
+from zope.interface import implementer
 from node.ext.ldap.interfaces import ILDAPProps
 
-MULTIVALUED_DEFAULTS = [
+
+MULTIVALUED_DEFAULTS = set([
     'member', 
     'uniqueMember', 
     'memberUid', 
     'memberOf'
-]
+])
 
-BINARY_DEFAULTS = [
+
+BINARY_DEFAULTS = set([
     # core.schema
     'userCertificate',
     'cACertificate',
@@ -23,14 +25,14 @@ BINARY_DEFAULTS = [
     'userPKCS12',
     'photo',
     'jpegPhoto',
-] 
+])
 
 
+@implementer(ILDAPProps)
 class LDAPServerProperties(object):
     """Wrapper Class for LDAP Server connection properties.
     """
-    implements(ILDAPProps)
-
+    
     def __init__(self,
                  server=None,
                  port=None,
@@ -46,7 +48,6 @@ class LDAPServerProperties(object):
                  #tls_clkeyfile=None,
                  retry_max=1,
                  retry_delay=10.0,
-                 escape_queries=False,
                  multivalued_attributes=MULTIVALUED_DEFAULTS,
                  binary_attributes=BINARY_DEFAULTS,
                  ):
@@ -108,16 +109,13 @@ class LDAPServerProperties(object):
         retry_delay
             Time span to wait between two reconnect trials
             Not yet
-        
-        escape_queries
-            Flag whether to escape queries
             
         multivalued_attributes
-            List of attributes names considered as multivalued to be returned 
+            Set of attributes names considered as multivalued to be returned
             as list. 
 
         binary_attributes
-            List of attributes names considered as binary. 
+            Set of attributes names considered as binary.
             (no unicode conversion)
         """
         if uri is None:
@@ -137,8 +135,7 @@ class LDAPServerProperties(object):
         #self.tls_clkeyfile = tls_clkeyfile
         self.retry_max = retry_max
         self.retry_delay = retry_delay
-        self.escape_queries = escape_queries
         self.multivalued_attributes = multivalued_attributes
         self.binary_attributes = binary_attributes
 
-LDAPProps = LDAPServerProperties
\ No newline at end of file
+LDAPProps = LDAPServerProperties
